@@ -1,5 +1,7 @@
-import unittest
 import io
+import unittest
+
+import pytest
 
 
 class TestHTTPChannel(unittest.TestCase):
@@ -29,13 +31,13 @@ class TestHTTPChannel(unittest.TestCase):
 
         inst, _, map = self._makeOneWithMap()
 
-        class DummyBuffer(object):
+        class DummyBuffer:
             chunks = []
 
             def append(self, data):
                 self.chunks.append(data)
 
-        class DummyData(object):
+        class DummyData:
             def __len__(self):
                 return MAXINT
 
@@ -173,7 +175,7 @@ class TestHTTPChannel(unittest.TestCase):
 
     def test_readable_with_requests(self):
         inst, sock, map = self._makeOneWithMap()
-        inst.requests = True
+        inst.requests = [True]
         self.assertEqual(inst.readable(), False)
 
     def test_handle_read_no_error(self):
@@ -189,13 +191,11 @@ class TestHTTPChannel(unittest.TestCase):
         self.assertEqual(L, [b"abc"])
 
     def test_handle_read_error(self):
-        import socket
-
         inst, sock, map = self._makeOneWithMap()
         inst.will_close = False
 
         def recv(b):
-            raise socket.error
+            raise OSError
 
         inst.recv = recv
         inst.last_activity = 0
@@ -270,7 +270,7 @@ class TestHTTPChannel(unittest.TestCase):
         class Lock(DummyLock):
             def wait(self):
                 inst.total_outbufs_len = 0
-                super(Lock, self).wait()
+                super().wait()
 
         inst.outbuf_lock = Lock()
         wrote = inst.write_soon(b"xyz")
@@ -367,7 +367,7 @@ class TestHTTPChannel(unittest.TestCase):
 
         inst, sock, map = self._makeOneWithMap()
 
-        class DummyHugeOutbuffer(object):
+        class DummyHugeOutbuffer:
             def __init__(self):
                 self.length = MAXINT + 1
 
@@ -439,7 +439,7 @@ class TestHTTPChannel(unittest.TestCase):
         preq.completed = False
         preq.empty = True
         inst.received(b"GET / HTTP/1.1\r\n\r\n")
-        self.assertEqual(inst.requests, ())
+        self.assertEqual(inst.requests, [])
         self.assertEqual(inst.server.tasks, [])
 
     def test_received_preq_completed_empty(self):
@@ -525,14 +525,6 @@ class TestHTTPChannel(unittest.TestCase):
         self.assertEqual(inst.sent_continue, True)
         self.assertEqual(preq.completed, False)
 
-    def test_service_no_requests(self):
-        inst, sock, map = self._makeOneWithMap()
-        inst.requests = []
-        inst.service()
-        self.assertEqual(inst.requests, [])
-        self.assertTrue(inst.server.trigger_pulled)
-        self.assertTrue(inst.last_activity)
-
     def test_service_with_one_request(self):
         inst, sock, map = self._makeOneWithMap()
         request = DummyRequest()
@@ -561,6 +553,7 @@ class TestHTTPChannel(unittest.TestCase):
         inst.task_class = DummyTaskClass()
         inst.requests = [request1, request2]
         inst.service()
+        inst.service()
         self.assertEqual(inst.requests, [])
         self.assertTrue(request1.serviced)
         self.assertTrue(request2.serviced)
@@ -705,7 +698,138 @@ class TestHTTPChannel(unittest.TestCase):
         self.assertEqual(inst.requests, [])
 
 
-class DummySock(object):
+class TestHTTPChannelLookahead(TestHTTPChannel):
+    def app_check_disconnect(self, environ, start_response):
+        """
+        Application that checks for client disconnection every
+        second for up to two seconds.
+        """
+        import time
+
+        if hasattr(self, "app_started"):
+            self.app_started.set()
+
+        try:
+            request_body_size = int(environ.get("CONTENT_LENGTH", 0))
+        except ValueError:
+            request_body_size = 0
+        self.request_body = environ["wsgi.input"].read(request_body_size)
+
+        self.disconnect_detected = False
+        check = environ["waitress.client_disconnected"]
+        if environ["PATH_INFO"] == "/sleep":
+            for i in range(3):
+                if i != 0:
+                    time.sleep(1)
+                if check():
+                    self.disconnect_detected = True
+                    break
+
+        body = b"finished"
+        cl = str(len(body))
+        start_response(
+            "200 OK", [("Content-Length", cl), ("Content-Type", "text/plain")]
+        )
+        return [body]
+
+    def _make_app_with_lookahead(self):
+        """
+        Setup a channel with lookahead and store it and the socket in self
+        """
+        adj = DummyAdjustments()
+        adj.channel_request_lookahead = 5
+        channel, sock, map = self._makeOneWithMap(adj=adj)
+        channel.server.application = self.app_check_disconnect
+
+        self.channel = channel
+        self.sock = sock
+
+    def _send(self, *lines):
+        """
+        Send lines through the socket with correct line endings
+        """
+        self.sock.send("".join(line + "\r\n" for line in lines).encode("ascii"))
+
+    def test_client_disconnect(self, close_before_start=False):
+        """Disconnect the socket after starting the task."""
+        import threading
+
+        self._make_app_with_lookahead()
+        self._send(
+            "GET /sleep HTTP/1.1",
+            "Host: localhost:8080",
+            "",
+        )
+        self.assertTrue(self.channel.readable())
+        self.channel.handle_read()
+        self.assertEqual(len(self.channel.server.tasks), 1)
+        self.app_started = threading.Event()
+        self.disconnect_detected = False
+        thread = threading.Thread(target=self.channel.server.tasks[0].service)
+
+        if not close_before_start:
+            thread.start()
+            self.assertTrue(self.app_started.wait(timeout=5))
+
+        # Close the socket, check that the channel is still readable due to the
+        # lookahead and read it, which marks the channel as closed.
+        self.sock.close()
+        self.assertTrue(self.channel.readable())
+        self.channel.handle_read()
+
+        if close_before_start:
+            thread.start()
+
+        thread.join()
+
+        if close_before_start:
+            self.assertFalse(self.app_started.is_set())
+        else:
+            self.assertTrue(self.disconnect_detected)
+
+    def test_client_disconnect_immediate(self):
+        """
+        The same test, but this time we close the socket even before processing
+        started. The app should not be executed.
+        """
+        self.test_client_disconnect(close_before_start=True)
+
+    def test_lookahead_continue(self):
+        """
+        Send two requests to a channel with lookahead and use an
+        expect-continue on the second one, making sure the responses still come
+        in the right order.
+        """
+        self._make_app_with_lookahead()
+        self._send(
+            "POST / HTTP/1.1",
+            "Host: localhost:8080",
+            "Content-Length: 1",
+            "",
+            "x",
+            "POST / HTTP/1.1",
+            "Host: localhost:8080",
+            "Content-Length: 1",
+            "Expect: 100-continue",
+            "",
+        )
+        self.channel.handle_read()
+        self.assertEqual(len(self.channel.requests), 1)
+        self.channel.server.tasks[0].service()
+        data = self.sock.recv(256).decode("ascii")
+        self.assertTrue(data.endswith("HTTP/1.1 100 Continue\r\n\r\n"))
+
+        self.sock.send(b"x")
+        self.channel.handle_read()
+        self.assertEqual(len(self.channel.requests), 1)
+        self.channel.server.tasks[0].service()
+        self.channel._flush_some()
+        data = self.sock.recv(256).decode("ascii")
+        self.assertEqual(data.split("\r\n")[-1], "finished")
+        self.assertEqual(self.request_body, b"x")
+
+
+class DummySock:
     blocking = False
     closed = False
 
@@ -731,8 +855,13 @@ class DummySock(object):
         self.sent += data
         return len(data)
 
+    def recv(self, buffer_size):
+        result = self.sent[:buffer_size]
+        self.sent = self.sent[buffer_size:]
+        return result
 
-class DummyLock(object):
+
+class DummyLock:
     notified = False
 
     def __init__(self, acquirable=True):
@@ -759,7 +888,7 @@ class DummyLock(object):
         pass
 
 
-class DummyBuffer(object):
+class DummyBuffer:
     closed = False
 
     def __init__(self, data, toraise=None):
@@ -783,7 +912,7 @@ class DummyBuffer(object):
         self.closed = True
 
 
-class DummyAdjustments(object):
+class DummyAdjustments:
     outbuf_overflow = 1048576
     outbuf_high_watermark = 1048576
     inbuf_overflow = 512000
@@ -796,11 +925,16 @@ class DummyAdjustments(object):
     expose_tracebacks = True
     ident = "waitress"
     max_request_header_size = 10000
+    url_prefix = ""
+    channel_request_lookahead = 0
+    max_request_body_size = 1048576
 
 
-class DummyServer(object):
+class DummyServer:
     trigger_pulled = False
     adj = DummyAdjustments()
+    effective_port = 8080
+    server_name = ""
 
     def __init__(self):
         self.tasks = []
@@ -813,7 +947,7 @@ class DummyServer(object):
         self.trigger_pulled = True
 
 
-class DummyParser(object):
+class DummyParser:
     version = 1
     data = None
     completed = True
@@ -831,7 +965,7 @@ class DummyParser(object):
         return len(data)
 
 
-class DummyRequest(object):
+class DummyRequest:
     error = None
     path = "/"
     version = "1.0"
@@ -844,7 +978,7 @@ class DummyRequest(object):
         self.closed = True
 
 
-class DummyLogger(object):
+class DummyLogger:
     def __init__(self):
         self.exceptions = []
         self.infos = []
@@ -857,13 +991,13 @@ class DummyLogger(object):
         self.exceptions.append(msg)
 
 
-class DummyError(object):
+class DummyError:
     code = "431"
     reason = "Bleh"
     body = "My body"
 
 
-class DummyTaskClass(object):
+class DummyTaskClass:
     wrote_header = True
     close_on_finish = False
     serviced = False
