@@ -1,5 +1,5 @@
-import unittest
 import io
+import unittest
 
 
 class TestThreadedTaskDispatcher(unittest.TestCase):
@@ -15,7 +15,7 @@ class TestThreadedTaskDispatcher(unittest.TestCase):
 
         class BadDummyTask(DummyTask):
             def service(self):
-                super(BadDummyTask, self).service()
+                super().service()
                 inst.stop_count += 1
                 raise Exception
 
@@ -85,7 +85,10 @@ class TestThreadedTaskDispatcher(unittest.TestCase):
         self.assertEqual(inst.shutdown(timeout=0.01), True)
         self.assertEqual(
             inst.logger.logged,
-            ["1 thread(s) still running", "Canceling 1 pending task(s)",],
+            [
+                "1 thread(s) still running",
+                "Canceling 1 pending task(s)",
+            ],
         )
         self.assertEqual(task.cancelled, True)
 
@@ -400,7 +403,7 @@ class TestWSGITask(unittest.TestCase):
         inst = self._makeOne()
 
         def execute():
-            raise socket.error
+            raise OSError
 
         inst.execute = execute
         self.assertRaises(socket.error, inst.service)
@@ -799,6 +802,7 @@ class TestWSGITask(unittest.TestCase):
                 "SERVER_PORT",
                 "SERVER_PROTOCOL",
                 "SERVER_SOFTWARE",
+                "waitress.client_disconnected",
                 "wsgi.errors",
                 "wsgi.file_wrapper",
                 "wsgi.input",
@@ -922,7 +926,7 @@ class TestErrorTask(unittest.TestCase):
         self.assertEqual(lines[8], b"(generated by waitress)")
 
 
-class DummyTask(object):
+class DummyTask:
     serviced = False
     cancelled = False
 
@@ -933,7 +937,7 @@ class DummyTask(object):
         self.cancelled = True
 
 
-class DummyAdj(object):
+class DummyAdj:
     log_socket_errors = True
     ident = "waitress"
     host = "127.0.0.1"
@@ -941,7 +945,7 @@ class DummyAdj(object):
     url_prefix = ""
 
 
-class DummyServer(object):
+class DummyServer:
     server_name = "localhost"
     effective_port = 80
 
@@ -949,12 +953,16 @@ class DummyServer(object):
         self.adj = DummyAdj()
 
 
-class DummyChannel(object):
+class DummyChannel:
     closed_when_done = False
     adj = DummyAdj()
     creation_time = 0
     addr = ("127.0.0.1", 39830)
 
+    def check_client_disconnected(self):
+        # For now, until we have tests handling this feature
+        return False
+
     def __init__(self, server=None):
         if server is None:
             server = DummyServer()
@@ -970,7 +978,7 @@ class DummyChannel(object):
         return len(data)
 
 
-class DummyParser(object):
+class DummyParser:
     version = "1.0"
     command = "GET"
     path = "/"
@@ -990,7 +998,7 @@ def filter_lines(s):
     return list(filter(None, s.split(b"\r\n")))
 
 
-class DummyLogger(object):
+class DummyLogger:
     def __init__(self):
         self.logged = []
 
